/**
 * @module botbuilder
 */
/**
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * Constants representing the API path that immediately follows the basePath.
 * These are currently internal but we will re-evaluate making them public, in the future.
 *
 * @example
 * RouteConstants.Activities = '/v3/conversations/:conversationId/activities'.
 */
export declare class RouteConstants {
    /**
     * Base API path for bot conversations.
     */
    static readonly Conversations: string;
    /**
     * API path for conversation history.
     */
    static readonly ConversationHistory: string;
    /**
     * API path for all conversation members.
     */
    static readonly ConversationMembers: string;
    /**
     * API path for page(s) of all conversation members.
     */
    static readonly ConversationPagedMembers: string;
    /**
     * API path for single conversation member.
     */
    static readonly ConversationMember: string;
    /**
     * API path for conversation attachments.
     */
    static readonly Attachments: string;
    /**
     * API path for all activities from conversation.
     */
    static readonly Activities: string;
    /**
     * API path for single activity from conversation.
     */
    static readonly Activity: string;
    /**
     * API path for all members from activity from conversation.
     */
    static readonly ActivityMembers: string;
}
//# sourceMappingURL=routeConstants.d.ts.map
