/**
 * @module botbuilder
 */
/**
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
import { Activity, OnBehalfOf, TeamInfo, TeamsMeetingInfo, TenantInfo } from 'botbuilder-core';
/**
 * Gets the Team's selected channel id from the current activity.
 *
 * @param activity The current [Activity](xref:botframework-schema.Activity).
 * @returns The current activity's team's selected channel, or empty string.
 */
export declare function teamsGetSelectedChannelId(activity: Activity): string;
/**
 * Gets the TeamsMeetingInfo object from the current [Activity](xref:botframework-schema.Activity).
 *
 * @param activity The current [Activity](xref:botframework-schema.Activity).
 * @returns The current [Activity](xref:botframework-schema.Activity)'s team meeting info, or null.
 */
export declare function teamsGetTeamMeetingInfo(activity: Activity): TeamsMeetingInfo | null;
/**
 * Gets the TenantInfo object from the current [Activity](xref:botframework-schema.Activity).
 *
 * @param activity The current [Activity](xref:botframework-schema.Activity).
 * @returns The current [Activity](xref:botframework-schema.Activity)'s tenant info, or null.
 */
export declare function teamsGetTenant(activity: Activity): TenantInfo | null;
/**
 * Gets the TeamsInfo object from the current [Activity](xref:botframework-schema.Activity).
 *
 * @param activity The current [Activity](xref:botframework-schema.Activity).
 * @returns The current [Activity](xref:botframework-schema.Activity)'s team's info, or null.
 */
export declare function teamsGetTeamInfo(activity: Activity): TeamInfo | null;
/**
 * Gets the Team Id from the current [Activity](xref:botframework-schema.Activity).
 *
 * @param activity The current [Activity](xref:botframework-schema.Activity).
 * @returns The current [Activity](xref:botframework-schema.Activity)'s team's Id, or null.
 */
export declare function teamsGetTeamId(activity: Activity): string | null;
/**
 * Activity helper methods for Teams.
 *
 * @param activity The current [Activity](xref:botframework-schema.Activity).
 * @returns The current [Activity](xref:botframework-schema.Activity)'s team's channel id, or null.
 */
export declare function teamsGetChannelId(activity: Activity): string | null;
/**
 * Configures the current [Activity](xref:botframework-schema.Activity) to generate a notification within Teams.
 *
 * @param activity The current [Activity](xref:botframework-schema.Activity).
 * @param alertInMeeting Sent to a meeting chat, this will cause the Teams client to render it in a notification popup as well as in the chat thread.
 * @param externalResourceUrl Url to external resource. Must be included in manifest's valid domains.
 */
export declare function teamsNotifyUser(activity: Partial<Activity>, alertInMeeting?: boolean, externalResourceUrl?: string): void;
/**
 * @param activity The current [Activity](xref:botframework-schema.Activity).
 * @returns The current [Activity](xref:botframework-schema.Activity)'s team's onBehalfOf list, or null.
 */
export declare function teamsGetTeamOnBehalfOf(activity: Activity): OnBehalfOf[];
//# sourceMappingURL=teamsActivityHelpers.d.ts.map